@@ -1,386 +1,13 @@
 #
-# t/test.pl - most of Test::More functionality without the fuss
+# t/test.pl - from CORE
 
-
-# NOTE:
-#
-# Increment ($x++) has a certain amount of cleverness for things like
-#
-#   $x = 'zz';
-#   $x++; # $x eq 'aaa';
-#
-# stands more chance of breaking than just a simple
-#
-#   $x = $x + 1
-#
-# In this file, we use the latter "Baby Perl" approach, and increment
-# will be worked over by t/op/inc.t
-
-$Level = 1;
-my $test = 1;
-my $planned;
-my $noplan;
-
-$TODO = 0;
-$NO_ENDING = 0;
-
-sub plan {
-    my $n;
-    if (@_ == 1) {
-	$n = shift;
-	if ($n eq 'no_plan') {
-	  undef $n;
-	  $noplan = 1;
-	}
-    } else {
-	my %plan = @_;
-	$n = $plan{tests};
-    }
-    print STDOUT "1..$n\n" unless $noplan;
-    $planned = $n;
-}
-
-END {
-    my $ran = $test - 1;
-    if (!$NO_ENDING) {
-	if (defined $planned && $planned != $ran) {
-	    print STDERR
-		"# Looks like you planned $planned tests but ran $ran.\n";
-	} elsif ($noplan) {
-	    print "1..$ran\n";
-	}
-    }
-}
-
-# Use this instead of "print STDERR" when outputing failure diagnostic
-# messages
-sub _diag {
-    return unless @_;
-    my @mess = map { /^#/ ? "$_\n" : "# $_\n" }
-               map { split /\n/ } @_;
-    my $fh = $TODO ? *STDOUT : *STDERR;
-    print $fh @mess;
-
-}
-
-sub diag {
-    _diag(@_);
-}
-
-sub skip_all {
-    if (@_) {
-	print STDOUT "1..0 # Skipped: @_\n";
-    } else {
-	print STDOUT "1..0\n";
-    }
-    exit(0);
-}
-
-sub _ok {
-    my ($pass, $where, $name, @mess) = @_;
-    # Do not try to microoptimize by factoring out the "not ".
-    # VMS will avenge.
-    my $out;
-    if ($name) {
-        # escape out '#' or it will interfere with '# skip' and such
-        $name =~ s/#/\\#/g;
-	$out = $pass ? "ok $test - $name" : "not ok $test - $name";
-    } else {
-	$out = $pass ? "ok $test" : "not ok $test";
-    }
-
-    $out .= " # TODO $TODO" if $TODO;
-    print STDOUT "$out\n";
-
-    unless ($pass) {
-	_diag "# Failed $where\n";
-    }
-
-    # Ensure that the message is properly escaped.
-    _diag @mess;
-
-    $test = $test + 1; # don't use ++
-
-    return $pass;
-}
+use Test::More;
 
 sub _where {
     my @caller = caller($Level);
     return "at $caller[1] line $caller[2]";
 }
 
-# DON'T use this for matches. Use like() instead.
-sub ok ($@) {
-    my ($pass, $name, @mess) = @_;
-    _ok($pass, _where(), $name, @mess);
-}
-
-sub _q {
-    my $x = shift;
-    return 'undef' unless defined $x;
-    my $q = $x;
-    $q =~ s/\\/\\\\/g;
-    $q =~ s/'/\\'/g;
-    return "'$q'";
-}
-
-sub _qq {
-    my $x = shift;
-    return defined $x ? '"' . display ($x) . '"' : 'undef';
-};
-
-# keys are the codes \n etc map to, values are 2 char strings such as \n
-my %backslash_escape;
-foreach my $x (split //, 'nrtfa\\\'"') {
-    $backslash_escape{ord eval "\"\\$x\""} = "\\$x";
-}
-# A way to display scalars containing control characters and Unicode.
-# Trying to avoid setting $_, or relying on local $_ to work.
-sub display {
-    my @result;
-    foreach my $x (@_) {
-        if (defined $x and not ref $x) {
-            my $y = '';
-            foreach my $c (unpack("U*", $x)) {
-                if ($c > 255) {
-                    $y .= sprintf "\\x{%x}", $c;
-                } elsif ($backslash_escape{$c}) {
-                    $y .= $backslash_escape{$c};
-                } else {
-                    my $z = chr $c; # Maybe we can get away with a literal...
-                    $z = sprintf "\\%03o", $c if $z =~ /[[:^print:]]/;
-                    $y .= $z;
-                }
-            }
-            $x = $y;
-        }
-        return $x unless wantarray;
-        push @result, $x;
-    }
-    return @result;
-}
-
-sub is ($$@) {
-    my ($got, $expected, $name, @mess) = @_;
-
-    my $pass;
-    if( !defined $got || !defined $expected ) {
-        # undef only matches undef
-        $pass = !defined $got && !defined $expected;
-    }
-    else {
-        $pass = $got eq $expected;
-    }
-
-    unless ($pass) {
-	unshift(@mess, "#      got "._q($got)."\n",
-		       "# expected "._q($expected)."\n");
-    }
-    _ok($pass, _where(), $name, @mess);
-}
-
-sub isnt ($$@) {
-    my ($got, $isnt, $name, @mess) = @_;
-
-    my $pass;
-    if( !defined $got || !defined $isnt ) {
-        # undef only matches undef
-        $pass = defined $got || defined $isnt;
-    }
-    else {
-        $pass = $got ne $isnt;
-    }
-
-    unless( $pass ) {
-        unshift(@mess, "# it should not be "._q($got)."\n",
-                       "# but it is.\n");
-    }
-    _ok($pass, _where(), $name, @mess);
-}
-
-sub cmp_ok ($$$@) {
-    my($got, $type, $expected, $name, @mess) = @_;
-
-    my $pass;
-    {
-        local $^W = 0;
-        local($@,$!);   # don't interfere with $@
-                        # eval() sometimes resets $!
-        $pass = eval "\$got $type \$expected";
-    }
-    unless ($pass) {
-        # It seems Irix long doubles can have 2147483648 and 2147483648
-        # that stringify to the same thing but are acutally numerically
-        # different. Display the numbers if $type isn't a string operator,
-        # and the numbers are stringwise the same.
-        # (all string operators have alphabetic names, so tr/a-z// is true)
-        # This will also show numbers for some uneeded cases, but will
-        # definately be helpful for things such as == and <= that fail
-        if ($got eq $expected and $type !~ tr/a-z//) {
-            unshift @mess, "# $got - $expected = " . ($got - $expected) . "\n";
-        }
-        unshift(@mess, "#      got "._q($got)."\n",
-                       "# expected $type "._q($expected)."\n");
-    }
-    _ok($pass, _where(), $name, @mess);
-}
-
-# Check that $got is within $range of $expected
-# if $range is 0, then check it's exact
-# else if $expected is 0, then $range is an absolute value
-# otherwise $range is a fractional error.
-# Here $range must be numeric, >= 0
-# Non numeric ranges might be a useful future extension. (eg %)
-sub within ($$$@) {
-    my ($got, $expected, $range, $name, @mess) = @_;
-    my $pass;
-    if (!defined $got or !defined $expected or !defined $range) {
-        # This is a fail, but doesn't need extra diagnostics
-    } elsif ($got !~ tr/0-9// or $expected !~ tr/0-9// or $range !~ tr/0-9//) {
-        # This is a fail
-        unshift @mess, "# got, expected and range must be numeric\n";
-    } elsif ($range < 0) {
-        # This is also a fail
-        unshift @mess, "# range must not be negative\n";
-    } elsif ($range == 0) {
-        # Within 0 is ==
-        $pass = $got == $expected;
-    } elsif ($expected == 0) {
-        # If expected is 0, treat range as absolute
-        $pass = ($got <= $range) && ($got >= - $range);
-    } else {
-        my $diff = $got - $expected;
-        $pass = abs ($diff / $expected) < $range;
-    }
-    unless ($pass) {
-        if ($got eq $expected) {
-            unshift @mess, "# $got - $expected = " . ($got - $expected) . "\n";
-        }
-	unshift@mess, "#      got "._q($got)."\n",
-		      "# expected "._q($expected)." (within "._q($range).")\n";
-    }
-    _ok($pass, _where(), $name, @mess);
-}
-
-# Note: this isn't quite as fancy as Test::More::like().
-
-sub like   ($$@) { like_yn (0,@_) }; # 0 for -
-sub unlike ($$@) { like_yn (1,@_) }; # 1 for un-
-
-sub like_yn ($$$@) {
-    my ($flip, $got, $expected, $name, @mess) = @_;
-    my $pass;
-    $pass = $got =~ /$expected/ if !$flip;
-    $pass = $got !~ /$expected/ if $flip;
-    unless ($pass) {
-	unshift(@mess, "#      got '$got'\n",
-		$flip
-		? "# expected !~ /$expected/\n" : "# expected /$expected/\n");
-    }
-    local $Level = $Level + 1;
-    _ok($pass, _where(), $name, @mess);
-}
-
-sub pass {
-    _ok(1, '', @_);
-}
-
-sub fail {
-    _ok(0, _where(), @_);
-}
-
-sub curr_test {
-    $test = shift if @_;
-    return $test;
-}
-
-sub next_test {
-  my $retval = $test;
-  $test = $test + 1; # don't use ++
-  $retval;
-}
-
-# Note: can't pass multipart messages since we try to
-# be compatible with Test::More::skip().
-sub skip {
-    my $why = shift;
-    my $n    = @_ ? shift : 1;
-    for (1..$n) {
-        print STDOUT "ok $test # skip: $why\n";
-        $test = $test + 1;
-    }
-    local $^W = 0;
-    last SKIP;
-}
-
-sub todo_skip {
-    my $why = shift;
-    my $n   = @_ ? shift : 1;
-
-    for (1..$n) {
-        print STDOUT "not ok $test # TODO & SKIP: $why\n";
-        $test = $test + 1;
-    }
-    local $^W = 0;
-    last TODO;
-}
-
-sub eq_array {
-    my ($ra, $rb) = @_;
-    return 0 unless $#$ra == $#$rb;
-    for my $i (0..$#$ra) {
-	next     if !defined $ra->[$i] && !defined $rb->[$i];
-	return 0 if !defined $ra->[$i];
-	return 0 if !defined $rb->[$i];
-	return 0 unless $ra->[$i] eq $rb->[$i];
-    }
-    return 1;
-}
-
-sub eq_hash {
-    my ($orig, $suspect) = @_;
-    my $fail;
-    while (my ($key, $value) = each %$suspect) {
-        # Force a hash recompute if this perl's internals can cache the hash key.
-        $key = "" . $key;
-        if (exists $orig->{$key}) {
-            if ($orig->{$key} ne $value) {
-                print STDOUT "# key ", _qq($key), " was ", _qq($orig->{$key}),
-                  " now ", _qq($value), "\n";
-                $fail = 1;
-            }
-        } else {
-            print STDOUT "# key ", _qq($key), " is ", _qq($value),
-              ", not in original.\n";
-            $fail = 1;
-        }
-    }
-    foreach (keys %$orig) {
-        # Force a hash recompute if this perl's internals can cache the hash key.
-        $_ = "" . $_;
-        next if (exists $suspect->{$_});
-        print STDOUT "# key ", _qq($_), " was ", _qq($orig->{$_}), " now missing.\n";
-        $fail = 1;
-    }
-    !$fail;
-}
-
-sub require_ok ($) {
-    my ($require) = @_;
-    eval <<REQUIRE_OK;
-require $require;
-REQUIRE_OK
-    _ok(!$@, _where(), "require $require");
-}
-
-sub use_ok ($) {
-    my ($use) = @_;
-    eval <<USE_OK;
-use $use;
-USE_OK
-    _ok(!$@, _where(), "use $use");
-}
-
 # runperl - Runs a separate perl interpreter.
 # Arguments :
 #   switches => [ command-line switches ]
@@ -405,7 +32,7 @@ sub _quote_args {
     foreach (@$args) {
 	# In VMS protect with doublequotes because otherwise
 	# DCL will lowercase -- unless already doublequoted.
-       $_ = q(").$_.q(") if $is_vms && !/^\"/ && length($_) > 0;
+        $_ = q(").$_.q(") if $is_vms && !/^\"/ && length($_) > 0;
 	$$runperl .= ' ' . $_;
     }
 }
@@ -503,7 +130,6 @@ sub runperl {
     die "test.pl:runperl() does not take a hashref"
 	if ref $_[0] and ref $_[0] eq 'HASH';
     my $runperl = &_create_runperl;
-    my $result;
     # ${^TAINT} is invalid in perl5.00505
     my $tainted;
     eval '$tainted = ${^TAINT};' if $] >= 5.006;
@@ -537,12 +163,14 @@ sub runperl {
 	$runperl =~ /(.*)/s;
 	$runperl = $1;
 
-	$result = `$runperl`;
+        my ($err,$result,$stderr) = run_cmd($runperl, $args{timeout});
+	$result =~ s/\n\n/\n/ if $is_vms; # XXX pipes sometimes double these
+	return $result;
     } else {
-	$result = `$runperl`;
+        my ($err,$result,$stderr) = run_cmd($runperl, $args{timeout});
+	$result =~ s/\n\n/\n/ if $is_vms; # XXX pipes sometimes double these
+	return $result;
     }
-    $result =~ s/\n\n/\n/ if $is_vms; # XXX pipes sometimes double these
-    return $result;
 }
 
 *run_perl = \&runperl; # Nice alias.
@@ -663,10 +291,10 @@ sub _fresh_perl {
 
     my $pass = $resolve->($results);
     unless ($pass) {
-        _diag "# PROG: \n$prog\n";
-        _diag "# EXPECTED:\n", $resolve->(), "\n";
-        _diag "# GOT:\n$results\n";
-        _diag "# STATUS: $status\n";
+        diag "# PROG: \n$prog\n";
+        diag "# EXPECTED:\n", $resolve->(), "\n";
+        diag "# GOT:\n$results\n";
+        diag "# STATUS: $status\n";
     }
 
     # Use the first line of the program as a name if none was given
@@ -675,7 +303,7 @@ sub _fresh_perl {
         $name .= '...' if length $first_line > length $name;
     }
 
-    _ok($pass, _where(), "fresh_perl - $name");
+    ok($pass, "fresh_perl - $name");
 }
 
 #
@@ -708,66 +336,48 @@ sub fresh_perl_like {
 		$runperl_args, $name);
 }
 
-sub can_ok ($@) {
-    my($proto, @methods) = @_;
-    my $class = ref $proto || $proto;
+# now my new B::C functions
 
-    unless( @methods ) {
-        return _ok( 0, _where(), "$class->can(...)" );
-    }
+sub run_cmd {
+    my ($cmd, $timeout) = @_;
 
-    my @nok = ();
-    foreach my $method (@methods) {
-        local($!, $@);  # don't interfere with caller's $@
-                        # eval sometimes resets $!
-        eval { $proto->can($method) } || push @nok, $method;
-    }
-
-    my $name;
-    $name = @methods == 1 ? "$class->can('$methods[0]')"
-                          : "$class->can(...)";
-
-    _ok( !@nok, _where(), $name );
-}
-
-sub isa_ok ($$;$) {
-    my($object, $class, $obj_name) = @_;
-
-    my $diag;
-    $obj_name = 'The object' unless defined $obj_name;
-    my $name = "$obj_name isa $class";
-    if( !defined $object ) {
-        $diag = "$obj_name isn't defined";
-    }
-    elsif( !ref $object ) {
-        $diag = "$obj_name isn't a reference";
+    my ($result, $out, $err) = (0, '', '');
+    if ( ! $have_IPC_Run ) {
+	local $@;
+	# No real way to trap STDERR?
+	$cmd .= " 2>&1" if($^O !~ /^MSWin32|VMS/);
+	$out = `$cmd`;
+	$result = $?;
     }
     else {
-        # We can't use UNIVERSAL::isa because we want to honor isa() overrides
-        local($@, $!);  # eval sometimes resets $!
-        my $rslt = eval { $object->isa($class) };
-        if( $@ ) {
-            if( $@ =~ /^Can't call method "isa" on unblessed reference/ ) {
-                if( !UNIVERSAL::isa($object, $class) ) {
-                    my $ref = ref $object;
-                    $diag = "$obj_name isn't a '$class' it's a '$ref'";
-                }
-            } else {
-                die <<WHOA;
-WHOA! I tried to call ->isa on your object and got some weird error.
-This should never happen.  Please contact the author immediately.
-Here's the error.
-$@
-WHOA
-            }
-        }
-        elsif( !$rslt ) {
-            my $ref = ref $object;
-            $diag = "$obj_name isn't a '$class' it's a '$ref'";
-        }
+	my $in;
+	my @cmd = split /\s+/, $cmd;
+	
+	eval {
+	    my $h = IPC::Run::start(\@cmd, \$in, \$out, \$err);
+	    if ($timeout) {
+		my $secs10 = $timeout/10;
+		for (1..$secs10) {
+		    if(!$h->pumpable) {
+			last;
+		    }
+		    else {
+			$h->pump_nb;
+			diag sprintf("waiting %d[s]",$_*10) if $_ > 30;
+			sleep 10;
+		    }
+		}
+		if($h->pumpable) {
+		    $h->kill_kill;
+		    $err .= "Timed out waiting for process exit";
+		}
+	    }
+	    $h->finish or die "cmd returned $?";
+	    $result = $h->result(0);
+	};
+	$err .= "\$\@ = $@" if($@);
     }
-
-    _ok( !$diag, _where(), $name );
+    return ($result, $out, $err);
 }
 
 sub tests {
@@ -787,6 +397,7 @@ sub run_cc_test {
     local($\, $,);   # guard against -l and other things that screw with
                      # print
     $expect =~ s/\n$//;
+    my ($out,$result,$stderr) = ('');
     my $fnbackend = lc($backend); #C,-O2
     ($fnbackend,$opt) = $fnbackend =~ /^(cc?)(,-o.)?/;
     $opt =~ s/,-/_/ if $opt;
@@ -815,6 +426,7 @@ sub run_cc_test {
                     verbose  => $ENV{TEST_VERBOSE}, # for debugging
                     nolib    => $ENV{PERL_CORE} ? 0 : 1, # include ../lib only in CORE
                     stderr   => 1, # to capture the "ccode.pl syntax ok"
+		    timeout  => 120,
                     progfile => $test);
     if (! $? and -s $cfile) {
         use ExtUtils::Embed ();
@@ -823,7 +435,7 @@ sub run_cc_test {
         $command .= " -lperl" if $command !~ /(-lperl|CORE\/libperl5)/ and $^O ne 'MSWin32';
         my $NULL = $^O eq 'MSWin32' ? '' : '2>/dev/null';
         my $cmdline = "$Config{cc} $command $NULL";
-        system($cmdline);
+        run_cmd($cmdline,20);
         unless (-e $exe) {
             print "not ok $cnt $todo failed $cmdline\n";
             print STDERR "# ",system("$Config{cc} $command"), "\n";
@@ -831,25 +443,29 @@ sub run_cc_test {
             return 0;
         }
         $exe = "./".$exe unless $^O eq 'MSWin32';
-        $got = `$exe`;
-        if (defined($got) and ! $?) {
+        ($result,$out,$stderr) = run_cmd($exe,5);
+        if (defined($out) and !$result) {
             if ($cnt == 25 and $expect eq '0 1 2 3 4321' and $] < 5.008) {
                 $expect = '0 1 2 3 4 5 4321';
             }
-            if ($got =~ /^$expect$/) {
+            if ($out =~ /^$expect$/) {
                 print "ok $cnt", $todo eq '#' ? "\n" : " $todo\n";
                 unlink ($test, $cfile, $exe, @obj) unless $keep_c;
                 return 1;
             } else {
-                print "not ok $cnt $todo wanted: \"$expect\", got: \"$got\"\n";
+                print "not ok $cnt $todo wanted: \"$expect\", got: \"$out\"\n";
                 unlink ($test, $cfile, $exe, @obj) unless $keep_c_fail;
                 return 0;
             }
         } else {
-            $got = '';
+            $out = '';
         }
     }
-    print "not ok $cnt $todo wanted: \"$expect\", \$\? = $?, got: \"$got\"\n";
+    print "not ok $cnt $todo wanted: \"$expect\", \$\? = $?, got: \"$out\"\n";
+    if ($stderr) {
+	$stderr =~ s/\n./\n# /xmsg;
+	print "# $stderr\n";
+    }
     unlink ($test, $cfile, $exe, @obj) unless $keep_c_fail;
     return 0;
 }
@@ -890,7 +506,7 @@ sub run_c_tests {
     # add some CC specific tests after 100
     # perl -lne "/^\s*sub pp_(\w+)/ && print \$1" lib/B/CC.pm > ccpp
     # for p in `cat ccpp`; do echo -n "$p "; grep -m1 " $p[(\[ ]" *.concise; done
-    # 
+    #
     # grep -A1 "coverage: ny" lib/B/CC.pm|grep sub
     # pp_stub pp_cond_expr pp_dbstate pp_reset pp_stringify pp_ncmp pp_preinc
     # pp_formline pp_enterwrite pp_leavewrite pp_entergiven pp_leavegiven
@@ -938,6 +554,7 @@ CCTESTS
             print sprintf("ok %d # skip\n", $cnt);
         } else {
             my ($script, $expect) = split />>>+\n/;
+	    die "Invalid empty t/TESTS" if !$script or $expect eq '';
             run_cc_test($cnt, $backend, $script, $expect, $keep_c, $keep_c_fail, $todo);
         }
         $cnt++;
